@@ -1,7 +1,7 @@
 """
 modify: functions that modify stored data without changing the data type
 Corey Rayburn Yung <coreyrayburnyung@gmail.com>
-Copyright 2020-2022, Corey Rayburn Yung
+Copyright 2020-2023, Corey Rayburn Yung
 License: Apache-2.0
 
     Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,34 +18,34 @@ License: Apache-2.0
 
 Contents:
     Adders:
-        add_prefix (Callable, dispatcher): adds a str prefix to item.
+        add_prefix (dispatcher): adds a str prefix to item.
         add_slots: adds '__slots__' to a dataclass.
-        add_suffix (Callable, dispatcher): adds a str suffix to item.
+        add_suffix (dispatcher): adds a str suffix to item.
     Dividers:
-        cleave (Callable, dispatcher): divides an item into 2 parts based on
+        cleave (dispatcher): divides an item into 2 parts based on
             'divider'.
-        separate (Callable, dispatcher): divides an item into n+1 parts based on
+        separate (dispatcher): divides an item into n+1 parts based on
             'divider'.
     Subtractors:
-        deduplicate (Callable, dispatcher): removes duplicate data from an item.
+        deduplicate (dispatcher): removes duplicate data from an item.
         drop_dunders: drops strings from a list if they start and end with 
             double underscores.
-        drop_prefix (Callable, dispatcher): removes a str prefix from an item.
+        drop_prefix (dispatcher): removes a str prefix from an item.
         drop_prefix_from_dict
         drop_prefix_from_list
         drop_prefix_from_set
         drop_prefix_from_str
         drop_prefix_from_tuple
         drop_privates
-        drop_substring (Callable, dispatcher): removes a substring from an item.
-        drop_suffix (Callable, dispatcher): removes a str suffix from an item.
+        drop_substring (dispatcher): removes a substring from an item.
+        drop_suffix (dispatcher): removes a str suffix from an item.
     Other: 
         capitalify: converts a snake case str to capital case.
         snakify: converts a capital case str to snake case.
         uniquify: returns a unique key for a dict.
 
 ToDo:
-    Reintegrate dispatcher from ashworth package once it has been tested.
+
 
 """
 from __future__ import annotations
@@ -60,26 +60,37 @@ from typing import Any, Optional, Type
 """ Adders """
 
 @functools.singledispatch
-def add_prefix(item: Any, prefix: str, divider: str = '') -> Any:
+def add_prefix(
+    item: Any, /, 
+    prefix: str, 
+    divider: Optional[str] = '',
+    recursive: Optional[bool] = False) -> Any:
     """Adds 'prefix' to 'item' with 'divider' in between.
     
     Args:
         item (Any): item to be modified.
         prefix (str): prefix to be added to 'item'.
-        divider (str): str to add between 'item' and 'prefix'. Defaults to '',
-            which means no divider will be added.
+        divider (Optional[str]): str to add between 'item' and 'prefix'. 
+            Defaults to '', which means no divider will be added.
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+
+    Returns:
+        Any: modified item.
 
     Raises:
         TypeError: if no registered function supports the type of 'item'.
         
-    Returns:
-        Any: modified item.
-
     """
     raise TypeError(f'item is not a supported type for {__name__}')
  
-@add_prefix.register
-def add_prefix_to_str(item: str, prefix: str, divider: str = '') -> str:
+@add_prefix.register(str)
+def add_prefix_to_str(
+    item: str,
+    prefix: str, 
+    divider: Optional[str] = '',
+    recursive: Optional[bool] = False) -> str:
     """Adds 'prefix' to 'item' with 'divider' in between.
     
     Args:
@@ -87,18 +98,23 @@ def add_prefix_to_str(item: str, prefix: str, divider: str = '') -> str:
         prefix (str): prefix to be added to 'item'.
         divider (str): str to add between 'item' and 'prefix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False. This argument has no effect
+            if 'item' is a str type.
+            
     Returns:
         str: modified str.
 
     """
     return divider.join([prefix, item])
  
-@add_prefix.register
+@add_prefix.register(Mapping)
 def add_prefix_to_dict(
-    item: Mapping[str, Any],  
+    item: Mapping[str, Any], /,
     prefix: str, 
-    divider: str = '') -> Mapping[str, Any]:
+    divider: Optional[str] = '',
+    recursive: Optional[bool] = False) -> Mapping[str, Any]:
     """Adds 'prefix' to keys in 'item' with 'divider' in between.
     
     Args:
@@ -106,25 +122,28 @@ def add_prefix_to_dict(
         prefix (str): prefix to be added to 'item'.
         divider (str): str to add between 'item' and 'prefix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+            
     Returns:
         Mapping[str, Any]: modified mapping.
 
     """
-    contents = {
-        add_prefix(item = k, prefix = prefix, divider = divider): v 
-        for k, v in item.items()}
-    if isinstance(item, dict):
-        return contents
+    base = type(item)
+    kwargs = dict(prefix = prefix, divider = divider, recursive = recursive)
+    if recursive:
+        tool = add_prefix
     else:
-        vessel = item.__class__
-        return vessel(contents)
+        tool = add_prefix_to_str
+    return base({tool(k, **kwargs): v for k, v in item.items()})
  
-@add_prefix.register
+@add_prefix.register(MutableSequence)
 def add_prefix_to_list(
-    item: MutableSequence[str], 
+    item: MutableSequence[str], /, 
     prefix: str, 
-    divider: str = '') -> MutableSequence[str]:
+    divider: str = '',
+    recursive: Optional[bool] = False) -> MutableSequence[str]:
     """Adds 'prefix' to items in 'item' with 'divider' in between.
     
     Args:
@@ -132,24 +151,28 @@ def add_prefix_to_list(
         prefix (str): prefix to be added to 'item'.
         divider (str): str to add between 'item' and 'prefix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+            
     Returns:
-        Any: modified sequence.
+        Any: modified mutable sequence.
 
     """
-    contents = [
-        add_prefix(item = i, prefix = prefix, divider = divider) for i in item]
-    if isinstance(item, list):
-        return contents
+    base = type(item)
+    kwargs = dict(prefix = prefix, divider = divider, recursive = recursive)
+    if recursive:
+        tool = add_prefix
     else:
-        vessel = item.__class__
-        return vessel(contents)
+        tool = add_prefix_to_str
+    return base([tool(i, **kwargs) for i in item])
  
-@add_prefix.register
+@add_prefix.register(Set)
 def add_prefix_to_set(
-    item: Set[str], 
+    item: Set[str], /, 
     prefix: str, 
-    divider: str = '') -> Set[str]:
+    divider: str = '',
+    recursive: Optional[bool] = False) -> Set[str]:
     """Adds 'prefix' to items in 'item' with 'divider' in between.
     
     Args:
@@ -157,24 +180,28 @@ def add_prefix_to_set(
         prefix (str): prefix to be added to 'item'.
         divider (str): str to add between 'item' and 'prefix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+            
     Returns:
         Set[str]: modified set.
 
     """
-    contents = {
-        add_prefix(item = i, prefix = prefix, divider = divider) for i in item}
-    if isinstance(item, set):
-        return contents
+    base = type(item)
+    kwargs = dict(prefix = prefix, divider = divider, recursive = recursive)
+    if recursive:
+        tool = add_prefix
     else:
-        vessel = item.__class__
-        return vessel(contents)
+        tool = add_prefix_to_str
+    return base({tool(i, **kwargs) for i in item})
 
-@add_prefix.register
+@add_prefix.register(tuple)
 def add_prefix_to_tuple(
-    item: tuple[str, ...], 
+    item: tuple[str, ...], /, 
     prefix: str, 
-    divider: str = '') -> tuple[str, ...]:
+    divider: str = '',
+    recursive: Optional[bool] = False) -> tuple[str, ...]:
     """Adds 'prefix' to items in 'item' with 'divider' in between.
     
     Args:
@@ -182,17 +209,19 @@ def add_prefix_to_tuple(
         prefix (str): prefix to be added to 'item'.
         divider (str): str to add between 'item' and 'prefix'. Defaults to '',
             which means no divider will be added.
-
-    Raises:
-        TypeError: if no registered function supports the type of 'item'.
-        
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+              
     Returns:
         tuple[str, ...]: modified tuple.
 
+    Raises:
+        TypeError: if no registered function supports the type of 'item'.
+      
     """
-    return tuple(
-        [add_prefix(item = i, prefix = prefix, divider = divider) 
-         for i in item])
+    kwargs = dict(prefix = prefix, divider = divider, recursive = recursive)
+    return tuple(add_prefix_to_list(item, **kwargs))
 
 def add_slots(item: Type[Any]) -> Type[Any]:
     """Adds slots to dataclass with default values.
@@ -202,13 +231,13 @@ def add_slots(item: Type[Any]) -> Type[Any]:
     
     Args:
         item (Type[Any]): dataclass to add slots to.
-        
-    Raises:
-        TypeError: if '__slots__' is already in item.
-        
+
     Returns:
         Type[Any]: class with '__slots__' added.
         
+    Raises:
+        TypeError: if '__slots__' is already in item.
+                
     """
     if '__slots__' in item.__dict__:
         raise TypeError(f'{item.__name__} already contains __slots__')
@@ -226,26 +255,37 @@ def add_slots(item: Type[Any]) -> Type[Any]:
     return item
 
 @functools.singledispatch
-def add_suffix(item: Any, suffix: str, divider: str = '') -> Any:
+def add_suffix(
+    item: Any, /, 
+    suffix: str, 
+    divider: Optional[str] = '',
+    recursive: Optional[bool] = False) -> Any:
     """Adds 'suffix' to 'item' with 'divider' in between.
     
     Args:
         item (Any): item to be modified.
         suffix (str): suffix to be added to 'item'.
-        divider (str): str to add between 'item' and 'suffix'. Defaults to '',
-            which means no divider will be added.
+        divider (Optional[str]): str to add between 'item' and 'suffix'. 
+            Defaults to '', which means no divider will be added.
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+
+    Returns:
+        Any: modified item.
 
     Raises:
         TypeError: if no registered function supports the type of 'item'.
         
-    Returns:
-        Any: modified item.
-
     """
     raise TypeError(f'item is not a supported type for {__name__}')
  
-@add_suffix.register
-def add_suffix_to_str(item: str, suffix: str, divider: str = '') -> str:
+@add_suffix.register(str)
+def add_suffix_to_str(
+    item: str,
+    suffix: str, 
+    divider: Optional[str] = '',
+    recursive: Optional[bool] = False) -> str:
     """Adds 'suffix' to 'item' with 'divider' in between.
     
     Args:
@@ -253,18 +293,23 @@ def add_suffix_to_str(item: str, suffix: str, divider: str = '') -> str:
         suffix (str): suffix to be added to 'item'.
         divider (str): str to add between 'item' and 'suffix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False. This argument has no effect
+            if 'item' is a str type.
+            
     Returns:
-        str: modified item.
+        str: modified str.
 
     """
     return divider.join([item, suffix])
  
-@add_suffix.register
+@add_suffix.register(Mapping)
 def add_suffix_to_dict(
-    item: Mapping[str, Any], 
+    item: Mapping[str, Any], /,
     suffix: str, 
-    divider: str = '') -> Mapping[str, Any]:
+    divider: Optional[str] = '',
+    recursive: Optional[bool] = False) -> Mapping[str, Any]:
     """Adds 'suffix' to keys in 'item' with 'divider' in between.
     
     Args:
@@ -272,25 +317,28 @@ def add_suffix_to_dict(
         suffix (str): suffix to be added to 'item'.
         divider (str): str to add between 'item' and 'suffix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+            
     Returns:
         Mapping[str, Any]: modified mapping.
 
     """
-    contents = {
-        add_suffix(item = k, suffix = suffix, divider = divider): v 
-        for k, v in item.items()}
-    if isinstance(item, dict):
-        return contents
+    base = type(item)
+    kwargs = dict(suffix = suffix, divider = divider, recursive = recursive)
+    if recursive:
+        tool = add_suffix
     else:
-        vessel = item.__class__
-        return vessel(contents)
+        tool = add_suffix_to_str
+    return base({tool(k, **kwargs): v for k, v in item.items()})
  
-@add_suffix.register
+@add_suffix.register(MutableSequence)
 def add_suffix_to_list(
-    item: MutableSequence[str], 
+    item: MutableSequence[str], /, 
     suffix: str, 
-    divider: str = '') -> MutableSequence[str]:
+    divider: str = '',
+    recursive: Optional[bool] = False) -> MutableSequence[str]:
     """Adds 'suffix' to items in 'item' with 'divider' in between.
     
     Args:
@@ -298,24 +346,28 @@ def add_suffix_to_list(
         suffix (str): suffix to be added to 'item'.
         divider (str): str to add between 'item' and 'suffix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+            
     Returns:
-        MutableSequence[str]: modified sequence.
+        Any: modified mutable sequence.
 
     """
-    contents = [
-        add_suffix(item = i, suffix = suffix, divider = divider) for i in item]
-    if isinstance(item, list):
-        return contents
+    base = type(item)
+    kwargs = dict(suffix = suffix, divider = divider, recursive = recursive)
+    if recursive:
+        tool = add_suffix
     else:
-        vessel = item.__class__
-        return vessel(contents)
+        tool = add_suffix_to_str
+    return base([tool(i, **kwargs) for i in item])
  
-@add_suffix.register
+@add_suffix.register(Set)
 def add_suffix_to_set(
-    item: Set[str], 
+    item: Set[str], /, 
     suffix: str, 
-    divider: str = '') -> Set[str]:
+    divider: str = '',
+    recursive: Optional[bool] = False) -> Set[str]:
     """Adds 'suffix' to items in 'item' with 'divider' in between.
     
     Args:
@@ -323,24 +375,28 @@ def add_suffix_to_set(
         suffix (str): suffix to be added to 'item'.
         divider (str): str to add between 'item' and 'suffix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+            
     Returns:
         Set[str]: modified set.
 
     """
-    contents = {add_suffix(item = i, suffix = suffix, divider = divider) 
-                for i in item}
-    if isinstance(item, set):
-        return contents
+    base = type(item)
+    kwargs = dict(suffix = suffix, divider = divider, recursive = recursive)
+    if recursive:
+        tool = add_suffix
     else:
-        vessel = item.__class__
-        return vessel(contents)
+        tool = add_suffix_to_str
+    return base({tool(i, **kwargs) for i in item})
 
-@add_suffix.register
+@add_suffix.register(tuple)
 def add_suffix_to_tuple(
-    item: tuple[str, ...], 
+    item: tuple[str, ...], /, 
     suffix: str, 
-    divider: str = '') -> tuple[str, ...]:
+    divider: str = '',
+    recursive: Optional[bool] = False) -> tuple[str, ...]:
     """Adds 'suffix' to items in 'item' with 'divider' in between.
     
     Args:
@@ -348,20 +404,25 @@ def add_suffix_to_tuple(
         suffix (str): suffix to be added to 'item'.
         divider (str): str to add between 'item' and 'suffix'. Defaults to '',
             which means no divider will be added.
-
+        recursive (Optional[bool]: False): if 'item' is nested, whether to apply
+            the function to all nested objects as well (True) or merely the top
+            level object (False). Defaults to False.
+              
     Returns:
         tuple[str, ...]: modified tuple.
 
+    Raises:
+        TypeError: if no registered function supports the type of 'item'.
+      
     """
-    return tuple(
-        [add_suffix(item = i, suffix = suffix, divider = divider) 
-         for i in item])
+    kwargs = dict(suffix = suffix, divider = divider, recursive = recursive)
+    return tuple(add_suffix_to_list(item, **kwargs))
 
 """ Dividers """
 
 @functools.singledispatch
 def cleave(
-    item: Any, 
+    item: Any, /, 
     divider: Any,
     return_last: bool = True,
     raise_error: bool = False) -> tuple[Any, Any]:
@@ -387,7 +448,7 @@ def cleave(
 
 @cleave.register
 def cleave_str(
-    item: str, 
+    item: str, /, 
     divider: str = '_',
     return_last: bool = True,
     raise_error: bool = False) -> tuple[str, str]:
@@ -423,7 +484,7 @@ def cleave_str(
 
 @functools.singledispatch
 def separate(
-    item: Any, 
+    item: Any, /, 
     divider: Any,
     raise_error: bool = False) -> tuple[Any, ...]:
     """Divides 'item' into n+1 parts based on 'divider'.
@@ -446,7 +507,7 @@ def separate(
 
 @separate.register
 def separate_str(
-    item: str, 
+    item: str, /, 
     divider: str = '_',
     raise_error: bool = False) -> list[str]:
     """Divides 'item' into n+1 parts based on 'divider'.
@@ -475,7 +536,7 @@ def separate_str(
 """ Subtractors """
 
 @functools.singledispatch
-def deduplicate(item: Any) -> Any:
+def deduplicate(item: Any, /) -> Any:
     """Deduplicates contents of 'item.
     
     Args:
@@ -490,8 +551,8 @@ def deduplicate(item: Any) -> Any:
     """
     raise TypeError(f'item is not a supported type for {__name__}')
 
-@deduplicate.register
-def deduplicate_list(item: MutableSequence[Any]) -> MutableSequence[Any]:
+@deduplicate.register(MutableSequence)
+def deduplicate_list(item: MutableSequence[Any], /) -> MutableSequence[Any]:
     """Deduplicates contents of 'item.
     
     Args:
@@ -501,15 +562,12 @@ def deduplicate_list(item: MutableSequence[Any]) -> MutableSequence[Any]:
         MutableSequence[Any]: deduplicated item.
         
     """
+    base = type(item)
     contents = list(dict.fromkeys(item))
-    if isinstance(item, list):
-        return contents
-    else:
-        vessel = item.__class__(contents)
-        return vessel(contents)
+    return base(contents)
 
-@deduplicate.register
-def deduplicate_tuple(item: tuple[Any, ...]) -> tuple[Any, ...]:
+@deduplicate.register(tuple)
+def deduplicate_tuple(item: tuple[Any, ...], /) -> tuple[Any, ...]:
     """Deduplicates contents of 'item.
     
     Args:
@@ -519,30 +577,74 @@ def deduplicate_tuple(item: tuple[Any, ...]) -> tuple[Any, ...]:
         tuple[Any, ...]: deduplicated item.
         
     """
-    return tuple(list(dict.fromkeys(item)))
-
-def drop_dunders(item: list[Any]) -> list[Any]:
-    """Drops items in 'item' with names beginning with an underscore.
-
-    Args:
-        item (list[Any]): attributes, methods, and properties of a class.
-
-    Returns:
-        list[Any]: attributes, methods, and properties that do not start with an
-            underscore.
-        
-    """
-    if len(item) > 0 and hasattr(item[0], '__name__'):
-        return [
-            i for i in item 
-            if not i.__name__.startswith('__') 
-            and not i.__name__.endswith('__')]
-    else:
-        return [
-            i for i in item if not i.startswith('__') and not i.endswith('__')]
+    return tuple(deduplicate_list(item))
     
 @functools.singledispatch
-def drop_prefix(item: Any, prefix: str, divider: str = '') -> Any:
+def drop_dunders(item: Any, /) -> Any:
+    """Drops items in 'item' beginning with a double underscore.
+
+    Args:
+        item (Any): item to modify.
+
+    Returns:
+        Any: item with entries dropped beginning with a double underscore.
+        
+    Raises:
+        TypeError: if 'item' is not a registered type.
+        
+    """
+    raise TypeError(f'item is not a supported type for {__name__}')
+
+@drop_dunders.register(Mapping)
+def drop_dunders_dict(item: Mapping[str, Any], /) -> Mapping[str, Any]:
+    """Drops items in 'item' beginning with a double underscore.
+
+    Args:
+        item (Mapping[str, Any]): dict-like object with str keys that might have
+            double underscores at the beginning of the key names.
+
+    Returns:
+        Mapping[str, Any]: dict-luke object with entries dropped if the key name
+            begin with a double underscore.
+        
+    """
+    base = type(item)
+    return base({k: v for k, v in item.items() if not k.startswith('__')})
+
+@drop_dunders.register(MutableSequence)   
+def drop_dunders_list(
+    item: MutableSequence[str | object], /) -> MutableSequence[str | object]:
+    """Drops items in 'item' beginning with a double underscore.
+
+    Args:
+        item (MutableSequence[str | object]): list-like object with str items or 
+            names that might have double underscores at their beginnings.
+
+    Returns:
+        MutableSequence[str | object]: list-like object with items dropped if 
+            they or their names begin with a double underscore.
+            
+    Raises:
+        TypeError: if 'item' does not contain str types or objects with either
+            'name' or '__name__' attributes.
+        
+    """
+    base = type(item)
+    if len(item) > 0 and all(isinstance(i, str) for i in item):
+        return base([i for i in item if not i.startswith('__')])
+    elif len(item) > 0 and all(hasattr(i, 'name') for i in item):
+        return base([i for i in item if not i.name.startswith('__')])
+    elif len(item) > 0 and all(hasattr(i, '__name__') for i in item):
+        return base([i for i in item if not i.__name__.startswith('__')])
+    elif len == 0:
+        return item
+    else:
+        raise TypeError(
+            'items in item must be str types or have name or __name__ '
+            'attributes')
+           
+@functools.singledispatch
+def drop_prefix(item: Any, /, prefix: str, divider: str = '') -> Any:
     """Drops 'prefix' from 'item' with 'divider' in between.
     
     Args:
@@ -561,7 +663,7 @@ def drop_prefix(item: Any, prefix: str, divider: str = '') -> Any:
     raise TypeError(f'item is not a supported type for {__name__}')
 
 @drop_prefix.register
-def drop_prefix_from_str(item: str, prefix: str, divider: str = '') -> str:
+def drop_prefix_from_str(item: str, /, prefix: str, divider: str = '') -> str:
     """Drops 'prefix' from 'item' with 'divider' in between.
     
     Args:
@@ -580,9 +682,9 @@ def drop_prefix_from_str(item: str, prefix: str, divider: str = '') -> str:
     else:
         return item
 
-@drop_prefix.register
+@drop_prefix.register(Mapping)
 def drop_prefix_from_dict(
-    item: Mapping[str, Any], 
+    item: Mapping[str, Any], /, 
     prefix: str, 
     divider: str = '') -> Mapping[str, Any]:
     """Drops 'prefix' from keys in 'item' with 'divider' in between.
@@ -606,9 +708,9 @@ def drop_prefix_from_dict(
         vessel = item.__class__
         return vessel(contents)
 
-@drop_prefix.register
+@drop_prefix.register(MutableSequence)
 def drop_prefix_from_list(
-    item: MutableSequence[str], 
+    item: MutableSequence[str], /, 
     prefix: str, 
     divider: str = '') -> MutableSequence[str]:
     """Drops 'prefix' from items in 'item' with 'divider' in between.
@@ -631,9 +733,9 @@ def drop_prefix_from_list(
         vessel = item.__class__
         return vessel(contents)
 
-@drop_prefix.register
+@drop_prefix.register(Set)
 def drop_prefix_from_set(
-    item: Set[str], 
+    item: Set[str], /, 
     prefix: str, 
     divider: str = '') -> Set[str]:
     """Drops 'prefix' from items in 'item' with 'divider' in between.
@@ -656,9 +758,9 @@ def drop_prefix_from_set(
         vessel = item.__class__
         return vessel(contents)  
 
-@drop_prefix.register
+@drop_prefix.register(tuple)
 def drop_prefix_from_tuple(
-    item: tuple[str, ...], 
+    item: tuple[str, ...], /, 
     prefix: str, 
     divider: str = '') -> tuple[str, ...]:
     """Drops 'prefix' from items in 'item' with 'divider' in between.
@@ -676,25 +778,73 @@ def drop_prefix_from_tuple(
     return tuple(
         [drop_prefix(item = i, prefix = prefix, divider = divider) 
          for i in item])       
-
-def drop_privates(item: list[Any]) -> list[Any]:
+    
+@functools.singledispatch
+def drop_privates(item: Any, /) -> Any:
     """Drops items in 'item' with names beginning with an underscore.
 
     Args:
-        item (list[Any]): attributes, methods, and properties of a class.
+        item (Any): item to modify.
 
     Returns:
-        list[Any]: attributes, methods, and properties that do not start with an
-            underscore.
+        Any: item with entries dropped beginning with an underscore.
+        
+    Raises:
+        TypeError: if 'item' is not a registered type.
         
     """
-    if len(item) > 0 and hasattr(item[0], '__name__'):
-        return [i for i in item if not i.__name__.startswith('_')]
+    raise TypeError(f'item is not a supported type for {__name__}')
+
+@drop_privates.register(Mapping)
+def drop_privates_dict(item: Mapping[str, Any], /) -> Mapping[str, Any]:
+    """Drops items in 'item' with key names beginning with an underscore.
+
+    Args:
+        item (Mapping[str, Any]): dict-like object with str keys that might have
+            underscores at the beginning of the key names.
+
+    Returns:
+        Mapping[str, Any]: dict-luke object with entries dropped if the key name
+            begin with an underscore.
+        
+    """
+    base = type(item)
+    return base({k: v for k, v in item.items() if not k.startswith('_')})
+
+@drop_privates.register(MutableSequence)   
+def drop_privates_list(
+    item: MutableSequence[str | object], /) -> MutableSequence[str | object]:
+    """Drops items in 'item' with names beginning with an underscore.
+
+    Args:
+        item (MutableSequence[str | object]): list-like object with str items or 
+            names that might have underscores at their beginnings.
+
+    Returns:
+        MutableSequence[str | object]: list-like object with items dropped if 
+            they or their names begin with an underscore.
+            
+    Raises:
+        TypeError: if 'item' does not contain str types or objects with either
+            'name' or '__name__' attributes.
+        
+    """
+    base = type(item)
+    if len(item) > 0 and all(isinstance(i, str) for i in item):
+        return base([i for i in item if not i.startswith('_')])
+    elif len(item) > 0 and all(hasattr(i, 'name') for i in item):
+        return base([i for i in item if not i.name.startswith('_')])
+    elif len(item) > 0 and all(hasattr(i, '__name__') for i in item):
+        return base([i for i in item if not i.__name__.startswith('_')])
+    elif len == 0:
+        return item
     else:
-        return [i for i in item if not i.startswith('_')]
-              
+        raise TypeError(
+            'items in item must be str types or have name or __name__ '
+            'attributes')
+                   
 @functools.singledispatch
-def drop_substring(item: Any, substring: str) -> Any:
+def drop_substring(item: Any, /, substring: str) -> Any:
     """Drops 'substring' from 'item' with a possible 'divider' in between.
     
     Args:
@@ -711,7 +861,7 @@ def drop_substring(item: Any, substring: str) -> Any:
     raise TypeError(f'item is not a supported type for {__name__}')
 
 @drop_substring.register
-def drop_substring_from_str(item: str, substring: str) -> str:
+def drop_substring_from_str(item: str, /, substring: str) -> str:
     """Drops 'substring' from 'item'.
     
     Args:
@@ -727,9 +877,9 @@ def drop_substring_from_str(item: str, substring: str) -> str:
     else:
         return item
 
-@drop_substring.register
+@drop_substring.register(Mapping)
 def drop_substring_from_dict(
-    item: Mapping[str, Any], 
+    item: Mapping[str, Any], /, 
     substring: str) -> Mapping[str, Any]:
     """Drops 'substring' from keys in 'item'.
     
@@ -750,9 +900,9 @@ def drop_substring_from_dict(
         vessel = item.__class__
         return vessel(contents)
 
-@drop_substring.register
+@drop_substring.register(MutableSequence)
 def drop_substring_from_list(
-    item: MutableSequence[str], 
+    item: MutableSequence[str], /, 
     substring: str) -> MutableSequence[str]:
     """Drops 'substring' from items in 'item'.
     
@@ -771,8 +921,8 @@ def drop_substring_from_list(
         vessel = item.__class__
         return vessel(contents)
 
-@drop_substring.register
-def drop_substring_from_set(item: Set[str], substring: str) -> Set[str]:
+@drop_substring.register(Set)
+def drop_substring_from_set(item: Set[str], /, substring: str) -> Set[str]:
     """Drops 'substring' from items in 'item'.
     
     Args:
@@ -790,9 +940,9 @@ def drop_substring_from_set(item: Set[str], substring: str) -> Set[str]:
         vessel = item.__class__
         return vessel(contents)  
 
-@drop_substring.register 
+@drop_substring.register(tuple)
 def drop_substring_from_tuple(
-    item: tuple[str, ...], 
+    item: tuple[str, ...], /, 
     substring: str) -> tuple[str, ...]:
     """Drops 'substring' from items in 'item'.
     
@@ -808,7 +958,7 @@ def drop_substring_from_tuple(
         [drop_substring(item = i, substring = substring) for i in item])    
      
 @functools.singledispatch
-def drop_suffix(item: Any, suffix: str, divider: str = '') -> Any:
+def drop_suffix(item: Any, /, suffix: str, divider: str = '') -> Any:
     """Drops 'suffix' from 'item' with 'divider' in between.
     
     Args:
@@ -825,7 +975,7 @@ def drop_suffix(item: Any, suffix: str, divider: str = '') -> Any:
     raise TypeError(f'item is not a supported type for {__name__}')
 
 @drop_suffix.register
-def drop_suffix_from_str(item: str, suffix: str, divider: str = '') -> str:
+def drop_suffix_from_str(item: str, /, suffix: str, divider: str = '') -> str:
     """Drops 'suffix' from 'item' with 'divider' in between.
     
     Args:
@@ -842,9 +992,9 @@ def drop_suffix_from_str(item: str, suffix: str, divider: str = '') -> str:
     else:
         return item
 
-@drop_suffix.register
+@drop_suffix.register(Mapping)
 def drop_suffix_from_dict(
-    item: Mapping[str, Any], 
+    item: Mapping[str, Any], /, 
     suffix: str, 
     divider: str = '') -> Mapping[str, Any]:
     """Drops 'suffix' from keys in 'item' with 'divider' in between.
@@ -866,9 +1016,9 @@ def drop_suffix_from_dict(
         vessel = item.__class__
         return vessel(contents)
 
-@drop_suffix.register
+@drop_suffix.register(MutableSequence)
 def drop_suffix_from_list(
-    item: MutableSequence[str], 
+    item: MutableSequence[str], /, 
     suffix: str, 
     divider: str = '') -> MutableSequence[str]:
     """Drops 'suffix' from items in 'item' with 'divider' in between.
@@ -889,9 +1039,9 @@ def drop_suffix_from_list(
         vessel = item.__class__
         return vessel(contents)
 
-@drop_suffix.register
+@drop_suffix.register(Set)
 def drop_suffix_from_set(
-    item: Set[str], 
+    item: Set[str], /, 
     suffix: str, 
     divider: str = '') -> Set[str]:
     """Drops 'suffix' from items in 'item' with 'divider' in between.
@@ -912,9 +1062,9 @@ def drop_suffix_from_set(
         vessel = item.__class__
         return vessel(contents)  
 
-@drop_suffix.register
+@drop_suffix.register(tuple)
 def drop_suffix_from_tuple(
-    item: tuple[str, ...], 
+    item: tuple[str, ...], /, 
     suffix: str, 
     divider: str = '') -> tuple[str, ...]:
     """Drops 'suffix' from items in 'item' with 'divider' in between.
